/*
 * $Header: /home/jerenkrantz/tmp/commons/commons-convert/cvs/home/cvs/jakarta-commons//resources/src/test/org/apache/commons/resources/impl/BasicMessageListTestCase.java,v 1.8 2004/03/27 21:34:09 dgraham Exp $
 * $Revision: 170280 $
 * $Date: 2005-05-15 19:32:46 -0400 (Sun, 15 May 2005) $
 *
 * ====================================================================
 *
 *  Copyright 2003-2004 The Apache Software Foundation
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package org.apache.commons.resources.impl;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.commons.resources.Message;
import org.apache.commons.resources.MessageList;

/**
 * Unit tests for the <code>org.apache.commons.resources.MessageList</code> class.
 * <p>
 * Originally based on org.apache.http.action.TestMessages, Revision 1.4.
 */
public class BasicMessageListTestCase extends TestCase {
	protected MessageList aMsgs = null;
	protected MessageList anMsgs = null;
	protected Message msg1 = null;
	protected Message msg2 = null;
	protected Message msg3 = null;
	protected Message msg4 = null;
	protected Message msg5 = null;
	protected Message msg6 = null;
	protected Message msg7 = null;

	/**
	 * Defines the testcase name for JUnit.
	 *
	 * @param theName the testcase's name.
	 */
	public BasicMessageListTestCase(String theName) {
		super(theName);
	}

	/**
	 * Start the tests.
	 *
	 * @param theArgs the arguments. Not used
	 */
	public static void main(String[] theArgs) {
		junit.awtui.TestRunner.main(new String[] { BasicMessageListTestCase.class.getName()});
	}

	/**
	 * @return a test suite (<code>TestSuite</code>) that includes all methods
	 *         starting with "test"
	 */
	public static Test suite() {
		// All methods starting with "test" will be executed in the test suite.
		return new TestSuite(BasicMessageListTestCase.class);
	}

	public void setUp() {
		aMsgs = new BasicMessageList();
		anMsgs = new BasicMessageList();
		Object[] objs1 = new Object[] { "a", "b", "c", "d", "e" };
		Object[] objs2 = new Object[] { "f", "g", "h", "i", "j" };
		msg1 = new BasicMessage("aMessage", objs1);
		msg2 = new BasicMessage("anMessage", objs2);
		msg3 = new BasicMessage("msg3", "value1");
		msg4 = new BasicMessage("msg4", "value2");
		msg5 = new BasicMessage("msg5", "value3", "value4");
		msg6 = new BasicMessage("msg5", "value3", "value4", "value5");
		msg7 = new BasicMessage("msg5", "value3", "value4", "value5", "value6");
	}

	public void tearDown() {
		aMsgs = null;
	}

	public void testEmpty() {
		assertTrue("aMsgs is not empty!", aMsgs.isEmpty());
	}

	public void testNotEmpty() {
		aMsgs.add("myProp", msg1);
		assertTrue("aMsgs is empty!", !aMsgs.isEmpty());
	}

	public void testSizeWithOneProperty() {
		aMsgs.add("myProp", msg1);
		aMsgs.add("myProp", msg2);
		assertEquals(2, aMsgs.size("myProp"));
	}

	public void testSizeWithManyProperties() {
		aMsgs.add("myProp1", msg1);
		aMsgs.add("myProp2", msg2);
		aMsgs.add("myProp3", msg3);
		aMsgs.add("myProp3", msg4);
		aMsgs.add("myProp4", msg5);
		assertEquals(1, aMsgs.size("myProp1"));
		assertEquals(5, aMsgs.size());
	}

	public void testSizeAndEmptyAfterClear() {
		testSizeWithOneProperty();
		aMsgs.clear();
		testEmpty();
		assertEquals(0, aMsgs.size("myProp"));
	}

	public void testGetWithNoProperty() {
		Iterator it = aMsgs.get("myProp");
		assertTrue("iterator is not empty!", !it.hasNext());
        assertTrue(aMsgs.isAccessed());
	}

	public void testGetForAProperty() {
		testSizeWithOneProperty();
		Iterator it = aMsgs.get("myProp");
		assertTrue("iterator is empty!", it.hasNext());
        assertTrue(aMsgs.isAccessed());
	}

	
	
	/**
	 * Tests using the globalMessageKey on the MessageList object.
	 */
	public void testGlobalMessageKey() {
		BasicMessage foo1 = new BasicMessage("key1");
		BasicMessage foo2 = new BasicMessage("key2");
		BasicMessage foo3 = new BasicMessage("key3");
		
		MessageList customKeyList = 
			new BasicMessageList("custom.global.message.key");
		
		MessageList standardKeyList = 
			new BasicMessageList();
		
		customKeyList.add("foo1", foo1);
		customKeyList.add("foo2", foo2);
		customKeyList.add(foo3);
		
		standardKeyList.add("foo1", foo1);
		standardKeyList.add(foo2);
		standardKeyList.add("foo3", foo3);
		
		assertTrue(customKeyList.size() == 3);
		assertTrue(standardKeyList.size() == 3);
		
		assertEquals(customKeyList.getGlobalMessageKey(), 
				"custom.global.message.key");
		
		assertEquals(standardKeyList.getGlobalMessageKey(), 
				MessageList.GLOBAL_MESSAGE_KEY);
		
		Iterator customKeyListIter = 
			customKeyList.get(MessageList.GLOBAL_MESSAGE_KEY);
		while (customKeyListIter.hasNext()) {
			fail("List should be empty");
		}
		
		Message msg = null;
		Iterator standardKeyListIter = 
			standardKeyList.get(MessageList.GLOBAL_MESSAGE_KEY);
		while (standardKeyListIter.hasNext()) {
			msg = (Message)standardKeyListIter.next();
			assertEquals(msg.getKey(), "key2");
		}
		msg = null;
				

		MessageList customKeyList2 = 
			new BasicMessageList("custom.global.message.key", customKeyList);
		
		MessageList standardKeyList2 = 
			new BasicMessageList(standardKeyList);
		
		Iterator customKeyList2Iter = 
			customKeyList2.get(MessageList.GLOBAL_MESSAGE_KEY);
		while (customKeyList2Iter.hasNext()) {
			fail("List should be empty");
		}
		
		Iterator standardKeyList2Iter = 
			standardKeyList2.get(MessageList.GLOBAL_MESSAGE_KEY);
		while (standardKeyList2Iter.hasNext()) {
			msg = (Message)standardKeyList2Iter.next();
			assertEquals(msg.getKey(), "key2");
		}
		msg = null;
		
		Map map = new HashMap();
		assertTrue(map.isEmpty());
		
		MessageList emptyList = new BasicMessageList();
		Iterator emptyListIter = emptyList.get();
		while (emptyListIter.hasNext()) {
			fail("List should be empty");
		}
		
		MessageList mList = new BasicMessageList("foo");
		assertEquals(mList.toString(), "{}");
				
	}
	
	
	
	/**
	 * Tests adding an MessageList object to an MessageList object.
	 */
	public void testAddMessages() {
		
		msg1 = new BasicMessage("key");
		msg2 = new BasicMessage("key2");
		msg3 = new BasicMessage("key3");
		MessageList outer = new BasicMessageList();
		MessageList inner = new BasicMessageList();

		outer.add("prop1", msg1);
		inner.add("prop1", msg2);
		inner.add("prop3", msg3);

		outer.add(inner);
		assertEquals(3, outer.size());
		assertEquals(2, outer.size("prop1"));

		// test message order
		Iterator props = outer.get();
		int count = 1;
		while (props.hasNext()) {
			Message msg = (Message) props.next();
			if (count == 1) {
				assertTrue(msg.getKey().equals("key"));
			} else if (count == 2) {
				assertTrue(msg.getKey().equals("key2"));
			} else {
				assertTrue(msg.getKey().equals("key3"));
			}
            count++;
			}
		}
}
