/*
 * $Header: /home/jerenkrantz/tmp/commons/commons-convert/cvs/home/cvs/jakarta-commons//resources/src/test/org/apache/commons/resources/impl/JDBCResourcesTestCase.java,v 1.2 2004/09/07 23:49:47 jmitchell Exp $
 * $Revision: 169946 $
 * $Date: 2005-05-13 01:15:41 -0400 (Fri, 13 May 2005) $
 *
 * ====================================================================
 *
 *  Copyright 2003-2004 The Apache Software Foundation
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
package org.apache.commons.resources.impl;

import java.io.File;
import java.io.FileReader;
import java.io.InputStream;
import java.net.URL;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;

import junit.framework.Test;
import junit.framework.TestSuite;

/**
 * <p>Unit tests for
 * <code>org.apache.commons.resources.impl.JDBCResources</code>.
 * </p>
 *
 * @author James Mitchell
 * @version $Revision: 169946 $ $Date: 2005-05-13 01:15:41 -0400 (Fri, 13 May 2005) $
 */

public class JDBCResourcesTestCase
    extends CollectionResourcesBaseTestCase {


    // ----------------------------------------------------- Instance Variables
	private Connection con;


    protected String getBase() throws Exception
    {
        URL url = 
            JDBCResourcesTestCase.class.getResource
            ("/org/apache/commons/resources/impl/jdbc.test.config.properties");
    
        if (url == null) {
            fail("URL NOT FOUND");
        }
        String string = url.toExternalForm();
        String base = string.substring(0, string.length() - 11);
        return base;
    }

    // ----------------------------------------------------------- Constructors


    public JDBCResourcesTestCase(String name) {
        super(name);
    }


    // --------------------------------------------------- Overall Test Methods


    // Set up instance variables required by this test case
    public void setUp() throws Exception {
        factory = new JDBCResourcesFactory();
        resources = factory.getResources(NAME, getBase());
		
//		InputStream in = this.getClass().getResourceAsStream("init.sql");
//        File file = new File("init.sql");
		
        // change this to load from init.sql
        String[] sql = {""
        + "create table resources (" 
        + "  locale            varchar(10)      not null,"
        + "  msgKey            varchar(255)     not null,"
        + "  val               varchar(255),"
        + "  Primary Key("
        + "    locale,"
        + "    msgKey"
        + "  )"
        + " );"
		, "Insert into resources (locale, msgKey, val) Values ('', 'test.base', '[Base] ONLY');"
        , "Insert into resources (locale, msgKey, val) Values ('', 'test.specific', '[Base] SPECIFIC');"
        , "Insert into resources (locale, msgKey, val) Values ('', 'test.inherit', '[Base] INHERIT');"
        , "Insert into resources (locale, msgKey, val) Values ('', 'test.message.single', '[Base] REPLACE {0}');"
        , "Insert into resources (locale, msgKey, val) Values ('', 'test.message', '[Base] REPLACE {0} WITH {1}');"
        , "Insert into resources (locale, msgKey, val) Values ('en', 'test.specific', '[en] SPECIFIC');"
        , "Insert into resources (locale, msgKey, val) Values ('en', 'test.inherit', '[en] INHERIT');"
        , "Insert into resources (locale, msgKey, val) Values ('en_US', 'test.specific', '[en_US] SPECIFIC');"
        , "Insert into resources (locale, msgKey, val) Values ('fr', 'test.specific', '[fr] SPECIFIC');"
        , "Insert into resources (locale, msgKey, val) Values ('fr', 'test.inherit', '[fr] INHERIT');"
        };
        runSql(sql);
    }

    // Return the tests included in this test suite
    public static Test suite() {
        return (new TestSuite(JDBCResourcesTestCase.class));
    }

    // Tear down the instance variables required by this test case
    public void tearDown() {
        resources = null;
        factory = null;
        try{
        	runSql(new String[]{"drop table resources"});
			con.close();
        }catch(Exception e){
        	// not really necessary to fail if creation also fails
        	e.printStackTrace();
        }

    }


    // ------------------------------------------------ Individual Test Methods


    // ------------------------------------------------------ Protected Methods


    // ------------------------------------------------------ Private Methods

    /**
	 * @param sql
	 * @throws SQLException
	 */
	private void runSql(String[] sql) throws SQLException {

		String url = "jdbc:hsqldb:.";
		
		Statement stmt;
		try {
			Class.forName("org.hsqldb.jdbcDriver");

		} catch(java.lang.ClassNotFoundException e) {
			fail("ClassNotFoundException:" + e.getMessage());
		}

		try {
			// TODO change this to pull from jdbc.test.config.properties
			if (con == null || con.isClosed())
				con = DriverManager.getConnection(url, "sa", "");
			stmt = con.createStatement();
			
			for (int i = 0; i < sql.length; i++) {
		   	    stmt.execute(sql[i]);
			}
			stmt.close();
	
		} catch(SQLException ex) {
			fail("SQLException: " + ex.getMessage());
		}	
    }
}
